#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            do {
                byte c;
                c = Char_Remain;
                repeat (2) {
                    repeat (2) {
                        *pVram = c;
                        ++c;
                        ++pVram;
                    }
                    pVram += VramRowSize - 2;
                }
                pVram += 2 - VramRowSize * 2;
                --i;
            } while (i > 0);
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	C
        0x9b, 0xa9, 0xa0, 0xa6, 
        0xa7, 0x9e, 0x9b, 0x9b, 
        0x9f, 0xa6, 0x9b, 0xa5, 
        0x9b, 0x9f, 0xa0, 0x9c, 
        //	A
        0x9b, 0xa9, 0xa8, 0x9d, 
        0xa7, 0x9e, 0x9b, 0xaa, 
        0xa7, 0xa2, 0xa0, 0xaa, 
        0x9f, 0x9c, 0x9b, 0xa0, 
        //	V
        0xa7, 0x9e, 0x9b, 0xaa, 
        0x9f, 0xa6, 0xa3, 0xa2, 
        0x9b, 0xa8, 0xa9, 0x9c, 
        0x9b, 0x9f, 0xa0, 0x9b, 
        //	I
        0x9f, 0xa8, 0xa2, 0x9c, 
        0x9b, 0xa7, 0x9e, 0x9b, 
        0x9b, 0xa7, 0x9e, 0x9b, 
        0x9f, 0xa0, 0xa0, 0x9c, 
        //	T
        0x9f, 0xa8, 0xa2, 0x9c, 
        0x9b, 0xa7, 0x9e, 0x9b, 
        0x9b, 0xa7, 0x9e, 0x9b, 
        0x9b, 0x9f, 0x9c, 0x9b, 
    };

    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (32 - 4 * 5) / 2;
        p = TitleBytes;
        repeat (5) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 20 + 7, "PUSH TRIGGER BUTTON");
}